@@ -4,6 +4,7 @@ mod ruleset;
 mod invocation;
 mod bank;
 mod symbol;
+mod function;
 
 
 pub mod parser;
@@ -32,4 +33,5 @@ pub use self::bank::BankData;
 pub use self::symbol::SymbolManager;
 pub use self::symbol::Symbol;
 pub use self::symbol::SymbolKind;
-pub use self::symbol::SymbolContext;
\ No newline at end of file
+pub use self::symbol::SymbolContext;
+pub use self::function::Function;
\ No newline at end of file
