@@ -8,6 +8,7 @@ mod symbol;
 mod data;
 mod addr_related;
 mod include;
+mod function;
 
 
 pub use self::state::State;
@@ -19,4 +20,5 @@ pub use self::rule_invocation::*;
 pub use self::symbol::*;
 pub use self::data::*;
 pub use self::addr_related::*;
-pub use self::include::*;
\ No newline at end of file
+pub use self::include::*;
+pub use self::function::*;
\ No newline at end of file
